package com.css.simulation.resource.common.controller;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.MinioParameter;
import api.common.util.FileUtil;
import com.css.simulation.resource.common.util.MinioUtil;
import io.minio.MinioClient;
import io.minio.errors.*;
import io.minio.http.Method;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

@RestController
@RequestMapping("/minio")
public class MinioController {

    @Resource
    private MinioClient minioClient;

    @Value("${minio.bucket-name}")
    private String bucketName;

    @Autowired
    RedisController redisController;


    @PostMapping("/hello")
    public String hello() {
        return "hello minio!";
    }

    @PostMapping(value = "/upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseBodyVO<String> upload(
            @RequestPart("file") MultipartFile file,
            @RequestParam("objectName") String objectName//  "/xx/xxx/x/xx"
    ) throws ServerException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
        MinioUtil.uploadFromMultipartFile(
                minioClient,
                file,
                bucketName,
                objectName
        );
        String previewUrl = MinioUtil.getPreviewUrl(minioClient, Method.GET, bucketName, objectName);

        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, "请求成功！", previewUrl);
    }


    @PostMapping(value = "/uploadProcessBar", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ResponseBodyVO<String> uploadProcessBar(
            @RequestPart("file") MultipartFile file,
            @RequestParam("objectName") String objectName,//  "/xx/xxx/x/xx"
            @RequestParam("objectPath") String objectPath
    ) throws ServerException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {

        MinioUtil.uploadProcessBar(
                minioClient,
                file,
                bucketName,
                objectName,
                objectPath,
                redisController
        );
        String previewUrl = MinioUtil.getPreviewUrl(minioClient, Method.GET, bucketName, objectName);

        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, "请求成功！", previewUrl);
    }

//    @PostMapping("/download")
//    public ResponseBodyVO<String> download(
//            @RequestBody @Validated MinioParameter minioParameter,
//            HttpServletResponse response
//    ) throws ServerException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
//        InputStream inputStream = MinioUtil.downloadToStream(
//                minioClient,
//                bucketName,
//                minioParameter.getObjectName()
//        );
//        String fileName = FileUtil.getFileName(minioParameter.getObjectName());
//        FileUtil.downloadForHttp(fileName, inputStream, response, 1024);
//        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);
//    }

    @PostMapping("/download")
    public void download(
            @RequestBody @Validated MinioParameter minioParameter,
            HttpServletResponse response
    ) throws ServerException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
        InputStream inputStream = MinioUtil.downloadToStream(
                minioClient,
                bucketName,
                minioParameter.getObjectName()
        );
        String fileName = FileUtil.getFileName(minioParameter.getObjectName());
        FileUtil.downloadForHttp(fileName, inputStream, response, 1024);
    }

    @RequestMapping("/preview")
    public void preview(@RequestParam("objectName") String objectName, HttpServletResponse response ) throws ServerException, InsufficientDataException, ErrorResponseException, IOException, NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
        InputStream inputStream = MinioUtil.downloadToStream(
                minioClient,
                bucketName,
                objectName
        );
        String fileName = FileUtil.getFileName(objectName);
        FileUtil.downloadForHttp(fileName, inputStream, response, 1024);
    }

}
