package com.css.simulation.resource.scheduler.scheduler;

import api.common.pojo.constants.DictConstants;
import api.common.util.TimeUtil;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
public class TickScheduler {

    @Value("${scheduler.manual-project.topic}")
    private String manualProjectTopic;
    @Autowired
    StringRedisTemplate redisTemplate;

    @Autowired
    private TaskMapper taskMapper;

    @Scheduled(fixedDelay = 2000)
    public void tick() {
        //1 查询出所有执行中的任务（除了 等待中 和 已完成）
        List<TaskPO> executingTaskList = taskMapper.selectExecuting();
        //2 根据 key 查出任务的心跳时间
        executingTaskList.forEach(task -> {
            String taskId = task.getId();
            String projectId = task.getPId();
            Long maxSimulationTime = task.getMaxSimulationTime();
            long tickTime = Long.parseLong(redisTemplate.opsForValue().get(manualProjectTopic + ":" + projectId + ":" + taskId));
            if (TimeUtil.getNow() - tickTime > maxSimulationTime) {
                //3 判断如果心跳时间距离当前时间已经超时，则修改任务状态为失败，并销毁 pod
                taskMapper.updateState(taskId, DictConstants.TASK_ABORTED);
            }
        });

    }
}
