package com.css.simulation.resource.algorithm.serviceImpl;
import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.algorithm.AlgorithmParameter;
import api.common.pojo.param.system.DictParam;
import api.common.pojo.po.algorithm.AlgorithmPO;
import api.common.pojo.constants.DictConstants;
import api.common.util.JgitUtil;
import api.common.util.StringUtil;
import api.common.pojo.vo.algorithm.AlgorithmVO;
import api.common.pojo.vo.algorithm.RunningProjectVO;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.algorithm.mapper.AlgorithmMapper;
import com.css.simulation.resource.algorithm.service.AlgorithmService;
import com.css.simulation.resource.system.service.DictService;
import com.github.pagehelper.PageInfo;
import org.eclipse.jgit.lib.Ref;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class AlgorithmServiceImpl implements AlgorithmService {

    @Autowired
    private AlgorithmMapper algorithmMapper;

    @Autowired
    DictService dictService;
    
    @Override
    public ResponseBodyVO addOrUpdate(AlgorithmParameter param) {

        String currentUserId = AuthUtil.getCurrentUserId();
        param.setCreateUserId(currentUserId);
        List<AlgorithmPO> algorithmPOS = algorithmMapper.selectAlgorithmName(param);
        if(algorithmPOS != null && algorithmPOS.size() > 0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"该算法名称已存在！");
        }else {
            if(isEmpty(param.getId())){
                //添加
                String algorithmId = StringUtil.getRandomUUID();
                param.setId(algorithmId);
                String algorithmCode = StringUtil.getRandomCode();
                param.setAlgorithmCode(algorithmCode);
                AlgorithmPO po = setPo(param, false);
                int add = algorithmMapper.add(po);
                if(add > 0){
                    return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
                }else {
                    return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"添加失败");
                }
            }else {
                //编辑
                AlgorithmPO po = setPo(param, true);
                int add = algorithmMapper.update(po);
                if (add > 0) {
                    return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
                }else {
                    return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "编辑失败");
                }
            }
        }
    }

    @Override
    public ResponseBodyVO selectAlgorithmList(AlgorithmParameter param) {
        param.setIsDeleted("0");
        param.setCreateUserId(AuthUtil.getCurrentUserId());
        PageUtil.setPageInfo(param);
        List<AlgorithmVO> vos = algorithmMapper.selectAlgorithmList(param);
        for (AlgorithmVO vo:vos){
            vo.setValidationStatus(getDictName(DictConstants.VALIDATION_STATUS, vo.getValidationStatus()));
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS,new PageInfo<>(vos));
    }

    @Override
    public ResponseBodyVO selectSharedAlgorithmList(AlgorithmParameter param) {
        param.setShare("1");
        PageUtil.setPageInfo(param);
        List<AlgorithmVO> vos = algorithmMapper.selectSharedAlgorithmList(param);
        for (AlgorithmVO vo:vos){
            vo.setValidationStatus(getDictName(DictConstants.VALIDATION_STATUS, vo.getValidationStatus()));
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS,new PageInfo<>(vos));
    }

    @Override
    public ResponseBodyVO shareAlgorithm(AlgorithmParameter param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }else {
            AlgorithmPO po = algorithmMapper.selectDetailsById(param);
            List<AlgorithmPO> algorithmPOS = algorithmMapper.selectSharedAlgorithmName(po.getAlgorithmName());
            if(algorithmPOS != null && algorithmPOS.size() > 0){
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"该算法名称已存在, 无法分享！");
            }else {
                po.setShare("1");
                po.setId(StringUtil.getRandomUUID());
                int add = algorithmMapper.add(po);
                if(add > 0){
                    return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,"分享成功");
                }
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"分享失败");

            }
        }
    }

    @Override
    public ResponseBodyVO testConnection(AlgorithmParameter param) {
        try{
            Map<String, Ref> asMap = JgitUtil.testConnectionByToken(param.getGitUrl(),param.getGitToken());
        } catch (Exception e){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"连接失败！");
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,"连接成功");
    }

    @Override
    public ResponseBodyVO isAlgorithmRunning(AlgorithmParameter param)
    {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }else {
            String algorithmId = param.getId();
            List<RunningProjectVO> runningProjectVos = algorithmMapper.selectRunningProject(algorithmId);
            if(runningProjectVos != null && runningProjectVos.size() > 0){
                StringBuffer stringBuffer = new StringBuffer("");
                for (RunningProjectVO vo:runningProjectVos){
                    String projectId = vo.getProjectId();
                    stringBuffer.append(projectId+",");
                }
                String substring = stringBuffer.substring(0, stringBuffer.lastIndexOf(","));
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,"已被工作ID："+ substring +" 应用，无法删除");
            }else {
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
            }

        }
    }

    @Override
    public ResponseBodyVO deleteByid(AlgorithmParameter param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }else {
            int i = algorithmMapper.deleteByid(param);
            if(i > 0){
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
            }
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"删除失败");
        }

    }

    @Override
    public ResponseBodyVO selectDetailsById(AlgorithmParameter param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }else {
            AlgorithmPO po = algorithmMapper.selectDetailsById(param);
            po.setValidationStatus(getDictName(DictConstants.VALIDATION_STATUS, po.getValidationStatus()));
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS,po);
        }

    }


    private boolean isEmpty(String value){
        if(value == null){
            return true;
        }
        value = value.trim();
        if(value.length() == 0){
            return true;
        }
        return false;
    }


    private AlgorithmPO setPo(AlgorithmParameter param,  Boolean isEdit) {

        AlgorithmPO po = new AlgorithmPO();
        //设置po信息
        if("2".equals(param.getUploadMode())){ //使用仓库地址上传
            String gitUrl = param.getGitUrl();
            String gitToken = param.getGitToken();
            po.setGitUrl(gitUrl);
            po.setGitToken(gitToken);
        }else {
            String minioPath = param.getMinioPath();
            String[] splits = minioPath.split("/");
            String fileName = splits[splits.length-1];
            po.setMinioPath(param.getMinioPath());
            po.setFileName(fileName);
        }
        po.setAlgorithmCode(param.getAlgorithmCode());
        po.setAlgorithmName(param.getAlgorithmName());
        po.setDescription(param.getDescription());
        po.setUploadMode(param.getUploadMode());
        po.setValidationStatus(DictConstants.SUCCESS);
        po.setShare("0");
        po.setId(param.getId());

        if(isEdit){
            po.setModifyTime(TimeUtil.getNowForMysql());
            po.setCreateUserId(AuthUtil.getCurrentUserId());
        }else {
            po.setIsDeleted("0");
            po.setCreateTime(TimeUtil.getNowForMysql());
            po.setModifyTime(TimeUtil.getNowForMysql());
            po.setCreateUserId(AuthUtil.getCurrentUserId());
            po.setModifyTime(TimeUtil.getNowForMysql());
        }
        return po;

    }

    private String getDictName(String type, String code) {
        DictParam dictParam = new DictParam();
        dictParam.setDictTypes(type);
        Map<String, Map<String, String>> dictMapsByTypes = dictService.getDictMapsByTypes(dictParam);
        Map<String, String> stringStringMap = dictMapsByTypes.get(type);
        if(stringStringMap != null && stringStringMap.size() > 0){
            return stringStringMap.get(code);
        }
        return "";
    }

    @Override
    public Map selectDetailsBySy() {
        AlgorithmParameter param=new AlgorithmParameter();
        param.setUploadMode("1");
        param.setShare("0");
        param.setCreateUserId(AuthUtil.getCurrentUserId());
        int fileNum= algorithmMapper.selectDetailsBySy(param);
        param.setUploadMode("2");
        int pathNum= algorithmMapper.selectDetailsBySy(param);
        Map map=new HashMap();
        map.put("fileNum",fileNum);
        map.put("pathNum",pathNum);
        return map;
    }

}
