package com.css.simulation.resource.project.ctrl;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.project.DictParam;
import api.common.pojo.param.project.SimulationManualProjectParam;
import com.css.simulation.resource.project.service.SimulationProjectService;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 工作台模块--项目运行
 */
@Controller
@RequestMapping("/simulationProject")
public class SimulationProjectCtrl {

    @Autowired
    private SimulationProjectService service;


    /**
     * 创建/修改工作
     * @param param
     * @return
     */
    @RequestMapping("addOrUpdateProject")
    @ResponseBody
    public ResponseBodyVO addOrUpdateProject(@RequestBody SimulationManualProjectParam param){
        return service.addOrUpdateProject(param);
    }

    /**
     * 查询项目列表
     * @param param
     * @return
     */
    @RequestMapping("selectProject")
    @ResponseBody
    public ResponseBodyVO selectProject(@RequestBody SimulationManualProjectParam param){
        return service.selectProject(param);
    }

    /**
     * 根据id查询
     * @return
     */
    @RequestMapping("selectProjectById")
    @ResponseBody
    public ResponseBodyVO selectProjectById(@RequestBody SimulationManualProjectParam param){
        return service.selectProjectById(param);
    }

    /**
     * 删除工作（支持批量删除）
     * @return
     */
    @RequestMapping("deleteProjectByids")
    @ResponseBody
    public ResponseBodyVO deleteProjectByids(@RequestBody SimulationManualProjectParam param){
        return service.deleteProjectByids(param);
    }

    /**
     * 修改工作运行状态
     * @param param
     * @return
     */
    @RequestMapping("updateProjectNowRunState")
    @ResponseBody
    public ResponseBodyVO updateProjectNowRunState(@RequestBody SimulationManualProjectParam param){
        return service.updateProjectNowRunState(param);
    }

    /**
     * 获取工作详情
     * @param param
     * @return
     */
    @RequestMapping("selectProjectDetailsById")
    @ResponseBody
    public ResponseBodyVO selectProjectDetailsById(@RequestBody SimulationManualProjectParam param){
        return service.selectProjectDetailsById(param);
    }

    /**
     * 获取测试报告详情
     * @param param
     * @return
     */
    @RequestMapping("selectProjectReportById")
    @ResponseBody
    public  ResponseBodyVO selectProjectReportById(@RequestBody SimulationManualProjectParam param){
        return service.selectProjectReportById(param);
    }

    /**
     * 获取任务详情信息
     * @param param
     * @return
     */
    @RequestMapping("selectProjectTaskById")
    @ResponseBody
    public ResponseBodyVO selectProjectTaskById(@RequestBody SimulationManualProjectParam param){
        return service.selectProjectTaskById(param);
    }

    /**
     * 获取添加工作时需要的下拉选项
     * @return
     */
    @RequestMapping("selectDropDownByType")
    @ResponseBody
    public ResponseBodyVO selectDropDownByType(@RequestBody SimulationManualProjectParam param){
        return service.selectDropDownByType(param);
    }


    /**
     * 获取人员可用最大并行度
     * @return
     */
    @RequestMapping("selectMaxParallelism")
    @ResponseBody
    public ResponseBodyVO selectMaxParallelism(){
        return service.selectMaxParallelism();
    }

    /**
     * 根据字典类型和code获取name（测试方法）
     * @param param
     * @return
     */
    @RequestMapping("getDictName")
    @ResponseBody
    public ResponseBodyVO getDictName(@RequestBody DictParam param) {
        String dictName = service.getDictName(param.getType(), param.getCode());
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,null,dictName);

    }


    /**
     * 解析车辆坐标文件
     * @return
     */
    @RequestMapping("analysisVehicleCoord")
    @ResponseBody
    public ResponseBodyVO analysisVehicleCoord() throws Exception {
        return service.analysisVehicleCoord();
    }


    /**
     * 查询工作任务列表
     * @param param
     * @return
     */
    @RequestMapping("selectProjectTaskList")
    @ResponseBody
    public ResponseBodyVO selectProjectTaskList(@RequestBody SimulationManualProjectParam param){
        return service.selectProjectTaskList(param);
    }




}
