package com.css.simulation.resource.project.impl;

import api.common.pojo.common.PageVO;
import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.KafkaParameter;
import api.common.pojo.param.project.ProjectTaskParam;
import api.common.pojo.param.project.SceneScoreParam;
import api.common.pojo.param.project.SimulationManualProjectKafkaParam;
import api.common.pojo.param.project.SimulationManualProjectParam;
import api.common.pojo.param.system.DictParam;
import api.common.pojo.po.algorithm.AlgorithmPO;
import api.common.pojo.po.project.*;
import api.common.pojo.po.scene.ScenePackageSublistPO;
import api.common.pojo.vo.project.*;
import api.common.pojo.vo.scene.RunProjectVO;
import api.common.util.JsonUtil;
import api.common.util.ObjectUtil;
import api.common.util.TimeUtil;
import com.alibaba.druid.support.spring.stat.SpringStatUtils;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.project.constants.ProjectConstants;
import com.css.simulation.resource.project.enums.ProjectRunStateEnum;
import com.css.simulation.resource.project.enums.SceneTypeEnum;
import com.css.simulation.resource.project.feign.KafkaService;
import com.css.simulation.resource.project.mapper.SimulationProjectMapper;
import com.css.simulation.resource.project.service.SimulationProjectService;
import com.css.simulation.resource.system.service.DictService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.github.pagehelper.PageInfo;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Slf4j
public class SimulationProjectServiceImpl implements SimulationProjectService {

    @Autowired
    private SimulationProjectMapper simulationProjectMapper;

    @Resource
    private KafkaService kafkaService;

    @Autowired
    DictService dictService;

    private String[] dateFmtArr= new String[]{"yyyyMMdd", "yyyy-MM-dd HH:mm:ss","yyyy-MM-dd"};

    @Override
    public ResponseBodyVO addOrUpdateProject(SimulationManualProjectParam param) {

        if(isEmpty(param.getProjectName())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE,"工作名称不能为空");
        }
        SimulationManualProjectPo po = convertParamToPo(param);

        if(isEmpty(param.getId())){

            //工作名称一样的的不能创建
            List<SimulationManualProjectPo> simulationManualProjectPos = simulationProjectMapper.selectProjectByName(param);
            if(!isEmpty(simulationManualProjectPos)){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE,"工作名称已经存在，请修改后再保存");
            }


            po.createPo(AuthUtil.getCurrentUserId());
            //生成id
            createProjectId(po);

            int add = simulationProjectMapper.add(po);
            if(add > 0){
                //第三方算法存到算发表(索为)
//                insertOtherAlgorithm(param);

                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,ResponseBodyVO.Response.SUCCESS.getMessage(),po.getId());
            }
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"添加失败");
        }else{
            //TODO 删除逻辑有待商议
            //获取工作状态，仅未开始的才可以修改信息
            SimulationManualProjectPo simulationManualProjectPo = simulationProjectMapper.selectProjectById(param);
            if(!ProjectRunStateEnum.NOT_START.getCode().equals(simulationManualProjectPo.getNowRunState())){
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"当前状态不支持修改");
            }

            po.updatePo(AuthUtil.getCurrentUserId());
            int update = simulationProjectMapper.update(po);
            if(update > 0){
                //第三方算法存到算发表(索为)
//                insertOtherAlgorithm(param);
                return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,ResponseBodyVO.Response.SUCCESS.getMessage(),po.getId());
            }

            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"修改失败");
        }

    }

    @Override
    public ResponseBodyVO selectProject(SimulationManualProjectParam param) {

        if(!isEmpty(param.getCreateStartDate())){
            String createStartDate = param.getCreateStartDate();
            Date startDate = getDate(createStartDate + " 00:00:00",1);
            if(startDate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setCreateTimeStart(startDate);
        }
        if(!isEmpty(param.getCreateEndDate())){
            String createEndDate = param.getCreateEndDate();
            Date endDate = getDate(createEndDate + " 23:59:59",1);
            if(endDate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setCreateTimeEnd(endDate);
        }

        if(!isEmpty(param.getFinishDateStart())){
            String finishDateStart = param.getFinishDateStart();
            Date startDate = getDate(finishDateStart + " 00:00:00",1);
            if(startDate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setFinishTimeStart(startDate);

        }
        if(!isEmpty(param.getFinishDateEnd())){
            String finishDateEnd = param.getFinishDateEnd();
            Date enddate = getDate(finishDateEnd + " 23:59:59",1);
            if(enddate == null){
                return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            param.setFinishTimeEnd(enddate);
        }

        //查询有效数据
        param.setIsDeleted("0");

        setPage(param.getCurrentPage()==null?1:param.getCurrentPage(), param.getPageSize()==null?10:param.getPageSize());
        List<SimulationManualProjectVo> vos = simulationProjectMapper.selectProjectList(param);
        PageInfo<SimulationManualProjectVo> pageInfo = new PageInfo(vos);

        for(SimulationManualProjectVo p : pageInfo.getList()){
            convertPoToVo(p);
        }

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,pageInfo);
    }

    @Override
    public ResponseBodyVO selectProjectById(SimulationManualProjectParam param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectById(param);
        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"没有获取到数据");
        }

        SimulationManualProjectSingleVo vo = new SimulationManualProjectSingleVo();
        convertPoToVo(po, vo);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,vo);

    }


    @Override
    public ResponseBodyVO deleteProjectByids(SimulationManualProjectParam param) {

        String ids = param.getIds();
        if(isEmpty(ids)){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }

        String[] idArr = ids.split(",");

        //执行中的不允许删除
        List<SimulationManualProjectPo> pos = simulationProjectMapper.selectProjectNowRunState(idArr);
        for(SimulationManualProjectPo p : pos){
            if(!isEmpty(p.getNowRunState()) && !ProjectRunStateEnum.NOT_START.getCode().equals(p.getNowRunState())){
                return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"数据不支持删除");
            }
        }

        int i = simulationProjectMapper.deleteProject(idArr);
        if(i > 0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }

        return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"删除失败");
    }

    @SneakyThrows
    @Override
    public ResponseBodyVO updateProjectNowRunState(SimulationManualProjectParam param) {
        log.info("updateProjectNowRunState_id:"+param.getId());
        log.info("updateProjectNowRunState_nowRunState:"+param.getNowRunState());

        SimulationManualProjectPo po = simulationProjectMapper.selectProjectById(param);

        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"数据没有找到");
        }

        if(isEmpty(param.getId()) || isEmpty(param.getNowRunState())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        if("20".equals(param.getNowRunState())){
            //设置开始时间
            param.setStartTime(new Date());

            //创建项目最后一级指标信息和场景信息（生成得分时创建数据）
//            createProjectSceneAndTarget(param.getId());
            ///TODO Kafka推送消息
            projectRunToKafka(po);


        }else if("30".equals(param.getNowRunState())){
            //设置完成时间
            param.setFinishTime(new Date());

        }
        int i = simulationProjectMapper.updateProjectNowRunState(param);
        if(i > 0){
            return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE);


    }

    private void projectRunToKafka(SimulationManualProjectPo po) throws JsonProcessingException {
        SimulationManualProjectKafkaParam kafkaParam = new SimulationManualProjectKafkaParam();
        kafkaParam.setProjectId(po.getId());
        kafkaParam.setAlgorithmId(po.getAlgorithm());
        kafkaParam.setVehicleConfigId(po.getVehicle());
        kafkaParam.setScenePackageId(po.getScene());
        kafkaParam.setMaxSimulationTime(po.getMaxSimulationTime());
        kafkaParam.setParallelism(Integer.valueOf(po.getParallelism()));
        KafkaParameter kafkaParameter = new KafkaParameter();
        kafkaParameter.setTopic(ProjectConstants.RUN_TASK_TOPIC);
        String data = JsonUtil.beanToJson(kafkaParam);
        kafkaParameter.setData(data);
        log.info("推送消息到kafka:"+data);
        kafkaService.send(kafkaParameter);
    }

    @Override
    public ResponseBodyVO selectProjectDetailsById(SimulationManualProjectParam param) {

        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        ProjectDetailsVo projectDetailsVo = new ProjectDetailsVo();



        //项目基本信息
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectById(param);
        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "没有获取到数据");
        }

        //评测等级
        String s = getEvaluationLevel(po);

        //算法配置
        String algorithmId = po.getAlgorithm();
        AlgorithmBaseInfoVo algorithmBaseInfoVo = simulationProjectMapper.selectAlgorithmBaseInfoById(algorithmId).get(0);

        //车辆配置
        VehicleBaseInfoVo vehicleBaseInfoVo = null;
        String vehicleConfigId = po.getVehicle();
        List<ConfigVehicleVO> configVehicleVOS = simulationProjectMapper.selectConfigVehicle(vehicleConfigId);
        if(!isEmpty(configVehicleVOS)){
            ConfigVehicleVO configVehicleVO = configVehicleVOS.get(0);
            vehicleBaseInfoVo = simulationProjectMapper.selectVehicleBaseInfoById(configVehicleVO.getVehicleId()).get(0);
        }
        String vehicleImage = vehicleBaseInfoVo.getVehicleImage();
        vehicleBaseInfoVo.setVehicleImage(vehicleImage.replaceFirst("Front side","Top"));

        List<VehicleSensorVo> vehicleSensorVos = simulationProjectMapper.selectVehicleSensor(vehicleConfigId);

        List<SensorBaseInfoVo> sensorCameraList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorOgtList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorLidarList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorRadarList = new ArrayList<>();
        List<SensorBaseInfoVo> sensorGpsList = new ArrayList<>();
        for(VehicleSensorVo vs : vehicleSensorVos){
            String sensorType = vs.getSensorType();
            String sensorId = vs.getSensorId();
            if(DictConstants.SENSOR_CAMERA.equals(sensorType)){
                //摄像头
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorCamera(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorCameraList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_OGT.equals(sensorType)){
                //完美传感器
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorOgt(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorOgtList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_LIDAR.equals(sensorType)){
                //激光雷达
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorLidar(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorLidarList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_RADAR.equals(sensorType)){
                //毫米波雷达
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorRadar(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorRadarList.add(sensorBaseInfoVo);
            }else if(DictConstants.SENSOR_GPS.equals(sensorType)){
                //gps
                SensorBaseInfoVo sensorBaseInfoVo = simulationProjectMapper.selectSensorGps(sensorId);
                setVehicleConfig(sensorBaseInfoVo,vs);
                sensorGpsList.add(sensorBaseInfoVo);
            }
        }

        //任务信息
        ProjectTaskParam projectTaskParam = new ProjectTaskParam();
        projectTaskParam.setPId(po.getId());
//        List<ManualProjectTaskPo> manualProjectTaskPos = simulationProjectMapper.selectprojectTaskByQuery(projectTaskParam);
//        List<ManualProjectTaskVo> ManualProjectTaskVoList = new ArrayList<>();
        List<ProjectRunStateNumVo> projectRunStateNumVoList = new ArrayList<>();
        List<ProjectRunResultRatioNumVo> projectRunResultRatioNumVoList = new ArrayList<>();
//        if(manualProjectTaskPos != null){
//            for(ManualProjectTaskPo task : manualProjectTaskPos){
//                ManualProjectTaskVo manualProjectTaskVo = convertManualProjectTaskPoToVo(task);
//                setUpSceneInfo(manualProjectTaskVo);
//                ManualProjectTaskVoList.add(manualProjectTaskVo);
//            }

            //任务运行状态统计
            Integer size = simulationProjectMapper.selectProjectTaskNumByProjectId(projectTaskParam);
//            List<ProjectRunStateNumVo> projectRunStateNumVos = simulationProjectMapper.selectRunStateCount(po.getId());
            projectRunStateNumVoList = simulationProjectMapper.selectRunStateCount(po.getId());

            /*状态统计统计数字，不统计比率
            for(ProjectRunStateNumVo pv : projectRunStateNumVos){
                Integer num = pv.getNum();
                Double d = (double)num/size;
                d = saveTwoDecimalPlaces(d);
                pv.setRatio(d);
                projectRunStateNumVoList.add(pv);
            }*/

            //结果状态统计
            List<ProjectRunResultRatioNumVo> projectRunResultRatioNumVos = simulationProjectMapper.selectRunResultCount(po.getId());
            for(ProjectRunResultRatioNumVo rv : projectRunResultRatioNumVos){
                Integer num = rv.getNum();
                Double d = (double)num/size;
                d = saveTwoDecimalPlaces(d);
                rv.setRatio(d);
                projectRunResultRatioNumVoList.add(rv);
            }

//        }



        projectDetailsVo.setProjectId(po.getProjectId());
        projectDetailsVo.setProjectName(po.getProjectName());
        projectDetailsVo.setProjectDescribe(po.getProjectDescribe());
        projectDetailsVo.setStartTime(getRqStr(po.getStartTime(),1));
        projectDetailsVo.setFinishTime(getRqStr(po.getFinishTime(),1));
        projectDetailsVo.setNowRunState(po.getNowRunState());
        projectDetailsVo.setNowRunStateName(getDictName(DictConstants.PROJECT_RUN_STATE, po.getNowRunState()));
//        projectDetailsVo.setEvaluationLevel(getDictName(DictConstants.EVALUATION_LEVEL, po.getEvaluationLevel()));
        projectDetailsVo.setEvaluationLevel(s);
        if(algorithmBaseInfoVo != null){
            projectDetailsVo.setAlgorithmName(algorithmBaseInfoVo.getAlgorithmName());
            projectDetailsVo.setAlgorithmDescribe(algorithmBaseInfoVo.getDescription());
        }
        if(vehicleBaseInfoVo != null){
            projectDetailsVo.setVehicleName(vehicleBaseInfoVo.getVehicleName());
            projectDetailsVo.setVehicleDescribe(vehicleBaseInfoVo.getDescription());
            projectDetailsVo.setVehicleImage(vehicleBaseInfoVo.getVehicleImage());

        }
        projectDetailsVo.setSensorCameraList(sensorCameraList);
        projectDetailsVo.setSensorOgtList(sensorOgtList);
        projectDetailsVo.setSensorLidarList(sensorLidarList);
        projectDetailsVo.setSensorRadarList(sensorRadarList);
        projectDetailsVo.setSensorGpsList(sensorGpsList);
//        projectDetailsVo.setTaskList(ManualProjectTaskVoList);
        projectDetailsVo.setStateList(projectRunStateNumVoList);
        projectDetailsVo.setResultList(projectRunResultRatioNumVoList);

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, projectDetailsVo);
    }

    private void setVehicleConfig(SensorBaseInfoVo vo, VehicleSensorVo vs){
        vo.setConfigId(vs.getConfigId());
        vo.setSensorId(vs.getSensorId());
        vo.setSensorType(vs.getSensorType());
        vo.setSensorX(saveTwoDecimalPlaces(vs.getSensorX()));
        vo.setSensorY(saveTwoDecimalPlaces(vs.getSensorY()));
        vo.setSensorZ(saveTwoDecimalPlaces(vs.getSensorZ()));
        vo.setSensorH(saveTwoDecimalPlaces(vs.getSensorH()));
        vo.setSensorP(saveTwoDecimalPlaces(vs.getSensorP()));
        vo.setSensorR(saveTwoDecimalPlaces(vs.getSensorR()));
        vo.setSensorPort(vs.getSensorPort());
    }



    @Override
    public ResponseBodyVO selectProjectReportById(SimulationManualProjectParam param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        //项目基本信息
        SimulationManualProjectPo po = simulationProjectMapper.selectProjectBaseById(param);
        if(po == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "没有获取到数据");
        }
        //算法配置
        String algorithmId = po.getAlgorithm();
        AlgorithmBaseInfoVo algorithmBaseInfoVo = simulationProjectMapper.selectAlgorithmBaseInfoById(algorithmId).get(0);


        ProjectReportVo projectReportVo = new ProjectReportVo();
        projectReportVo.setAlgorithmName(algorithmBaseInfoVo.getAlgorithmName());
//        Double algorithmScore = po.getAlgorithmScore();
//        projectReportVo.setAlgorithmScore(algorithmScore);
        projectReportVo.setAlgorithmDescribe(algorithmBaseInfoVo.getDescription());

//        String evaluationLevelReport = getEvaluationLevelReport(algorithmScore);
//        projectReportVo.setEvaluationLevel(evaluationLevelReport);


        String sceneNames = "";

        //算法测试得分表
        /*
        汇总测试得分计算方法：（每一项一级指标的测试得分*测试权重）累加
        得分率计算方法：每一项一级指标的所有场景得分不为0的数量/每一项一级指标的场景数量
        汇总得分率计算方法：每一项一级指标的所有场景得分不为0的数量_累加/每一项一级指标的场景数量_累加
         */
        List<ScenePackageSubListVO> scenePackageSubListVOS = simulationProjectMapper.selectSubSceneByPid(po.getScene());

        List<AlgorithmScoreVo> algorithmScoreVoList = new ArrayList();
        String evaluationLevelReport = "";
        if(!isEmpty(scenePackageSubListVOS)){
            StringBuffer stringBuffer = new StringBuffer("");

            //汇总数据初始化
            Integer totalSceneNum = 0;
//            Double totalWeight = 0D;
            Double totalScore = 0D;
            Double totalScoreRatio=0D;
            Integer totalSceneScoreNum = 0;

            for(ScenePackageSubListVO v : scenePackageSubListVOS){
                stringBuffer.append(v.getSublistName()+"、");
                AlgorithmScoreVo algorithmScoreVo = new AlgorithmScoreVo();
                algorithmScoreVo.setProjectName(v.getSublistName());
                Integer sceneNum = v.getSceneNum();
                totalSceneNum += sceneNum;
                algorithmScoreVo.setSceneNum(sceneNum);
                String weight = v.getWeight();
                Double weightDouble = Double.valueOf(weight);
                algorithmScoreVo.setWeight(weight);
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo = new SimulationMptFirstTargetScorePo();
                simulationMptFirstTargetScorePo.setPId(po.getId());
                simulationMptFirstTargetScorePo.setTarget(v.getSublistId());
                //单个二级指标得分
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo1 = simulationProjectMapper.selectFirstTargetScore(simulationMptFirstTargetScorePo);
                Double score = simulationMptFirstTargetScorePo1.getScore();
                totalScore += score*(weightDouble/100);
                algorithmScoreVo.setScore(saveTwoDecimalPlaces(score));
//                totalWeight +=aDouble;
//                Integer num = v.getSceneNum();
                //获取得分不为0的场景数量
                Integer scoreNum = getSetScoreNum(v.getSublistId(),po.getId());
                totalSceneScoreNum += scoreNum;
                algorithmScoreVo.setScoreRatio(saveTwoDecimalPlaces(Double.valueOf(scoreNum)/Double.valueOf(sceneNum)*100));
                algorithmScoreVoList.add(algorithmScoreVo);
            }
            //汇总
            AlgorithmScoreVo algorithmScoreVo = new AlgorithmScoreVo();
            algorithmScoreVo.setProjectName("汇总");
            algorithmScoreVo.setSceneNum(totalSceneNum);
//            algorithmScoreVo.setWeight(saveTwoDecimalPlaces(totalWeight).toString());
            //指标权重总和默认是100%
            algorithmScoreVo.setWeight("100");
            algorithmScoreVo.setScore(saveTwoDecimalPlaces(totalScore));
            totalScoreRatio = Double.valueOf(totalSceneScoreNum)/Double.valueOf(totalSceneNum)*100;
            algorithmScoreVo.setScoreRatio(saveTwoDecimalPlaces(totalScoreRatio));
            algorithmScoreVoList.add(algorithmScoreVo);

            projectReportVo.setScoreRatio(saveTwoDecimalPlaces(totalScoreRatio));

            projectReportVo.setAlgorithmScore(saveTwoDecimalPlaces(totalScore));
            evaluationLevelReport = getEvaluationLevelReport(saveTwoDecimalPlaces(totalScore));
            projectReportVo.setEvaluationLevel(evaluationLevelReport);

            sceneNames = stringBuffer.substring(0,stringBuffer.lastIndexOf("、"));
        }



        projectReportVo.setAlgorithmEvaluation(algorithmBaseInfoVo.getAlgorithmName()+"经测试获得"+evaluationLevelReport+"级评价，"+
                "("+sceneNames+")得分率达到了"+projectReportVo.getScoreRatio()+"。");

        projectReportVo.setAlgorithmScoreList(algorithmScoreVoList);
        projectReportVo.setEvaluationGrade(evaluationLevelReport);

        //算法测试评分细则、详细得分情况
        /*List<ScenePackageSubListVO> scenePackageSubListVOS1 = selectScenePackageSubListTreeAndSetScore(null, true, po.getScene(), po.getId(), 2);
        projectReportVo.setSceneScoreList(scenePackageSubListVOS1);*/

        //查询场景包所有数据
//        List<SublistScoreVo> pos = simulationProjectMapper.selectSubScore(po.getScene());

        //指标得分列表
        Map<String, Object> stringObjectMap = selectScenePackageSubListAndSetScore(po.getScene());
        projectReportVo.setSubListScoreLiTitle((List<Map>) stringObjectMap.get("cloums"));
        projectReportVo.setSubListScoreLi((List<SubScListVo>) stringObjectMap.get("result"));

        //场景得分列表
//        Map<String, Object> maps = selectSceneScore(pos, po.getId());
        Map<String, Object> maps = selectSceneScore(po.getScene(), po.getId());
        projectReportVo.setSceneScoreLiTitle((List<Map>) maps.get("cloums"));
        projectReportVo.setSceneScoreLi((List<SublistScoreVo>) maps.get("result"));

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, projectReportVo);
    }




    @Override
    public ResponseBodyVO selectDropDownByType(SimulationManualProjectParam param) {
        List<DropDownTypeVo> result = new ArrayList<>();
        if(isEmpty(param.getDropDownType())){
            //获取全部（算法，车辆，场景）
            setAlgorithmDropDown(result);
            setVehicleDropDown(result);
            setScenePackageDropDown(result);
        }else if("1".equals(param.getDropDownType())){
            setAlgorithmDropDown(result);
        }else if("2".equals(param.getDropDownType())){
            setVehicleDropDown(result);
        }else if("3".equals(param.getDropDownType())){
            setScenePackageDropDown(result);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, result);
    }

    @Override
    public ResponseBodyVO selectProjectTaskById(SimulationManualProjectParam param) {
        String id = param.getId();
        String taskId = param.getTaskId();
        if(isEmpty(id) || isEmpty(taskId)){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }

        ProjectTaskDetailsVo resultVo = new ProjectTaskDetailsVo();

        //基本信息
        ProjectTaskParam projectTaskParam = new ProjectTaskParam();
        projectTaskParam.setPId(id);
        projectTaskParam.setId(taskId);
        List<ManualProjectTaskPo> pos = simulationProjectMapper.selectprojectTaskByQuery(projectTaskParam);
        if(isEmpty(pos)){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"没有获取到任务信息");
        }
        ManualProjectTaskPo po = pos.get(0);
        resultVo.setTaskId(po.getId());
        String sceneId = po.getSceneId();
        String sceneType = po.getSceneType();
        SceneBaseInfoVo sceneBaseInfoVo = getsceneNameAndOther(sceneId, sceneType);
        resultVo.setSceneName(sceneBaseInfoVo.getCommonSceneName());
        resultVo.setRunStartTime(getRqStr(po.getRunStartTime(),1));
        resultVo.setRunEndTime(getRqStr(po.getRunEndTime(),1));
        resultVo.setRunState(po.getRunState());
        resultVo.setFinishState(po.getRunResult());
        resultVo.setSceneDescribe("");
        ///TODO 任务详情信息
        resultVo.setMileage(po.getMileage());
        resultVo.setAverageVelocity(po.getAverageVelocity());
        resultVo.setMaximunSpeed(po.getMaximunSpeed());
        resultVo.setMinimunVelocity(po.getMinimunVelocity());
        resultVo.setMaximumDeceleration(po.getMaximumDeceleration());
        resultVo.setMaximumSwingSpeed(po.getMaximumSwingSpeed());
        resultVo.setSpeedVariance(saveTwoDecimalPlaces(po.getSpeedVariance()));
        resultVo.setSpeedComfortLevel(po.getSpeedComfortLevel());
        resultVo.setSwingSpeedMeanSquareRoot(saveTwoDecimalPlaces(po.getSwingSpeedMeanSquareRoot()));
        resultVo.setSwingComfortLevel(po.getSwingComfortLevel());
        resultVo.setAcceleration(po.getAcceleration());
        resultVo.setLaneOffset(po.getLaneOffset());
        resultVo.setBrake(po.getBrake());
        resultVo.setSteetingWheel(po.getSteetingWheel());
        resultVo.setThrottle(po.getThrottle());
        resultVo.setYawRate(po.getYawRate());
        resultVo.setVelocity(po.getVelocity());
        resultVo.setOtherCurve(po.getOtherCurve());

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,resultVo);
    }

    @Override
    public ResponseBodyVO selectMaxParallelism() {
        String currentUserId = AuthUtil.getCurrentUserId();
        SystemUserVo systemUserVo = simulationProjectMapper.selectUserById(currentUserId);
        if(systemUserVo == null){
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE,"没有获取到用户信息");
        }

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,null,systemUserVo.getCpuAvailableNumber());
    }

    @Override
    public String getDictName(String type, String code) {
        DictParam dictParam = new DictParam();
        dictParam.setDictTypes(type);
        Map<String, Map<String, String>> dictMapsByTypes = dictService.getDictMapsByTypes(dictParam);
        Map<String, String> stringStringMap = dictMapsByTypes.get(type);
        if(stringStringMap != null && stringStringMap.size() > 0){
            return stringStringMap.get(code);
        }
        return "";
    }

    @Override
    public ResponseBodyVO analysisVehicleCoord() throws Exception {

        analysisCsvFile("E:\\仿真云平台\\三维还原视频说明\\360ogt.csv",new int[]{6,7,9});

        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,null);

    }

    @Override
    public ResponseBodyVO selectProjectTaskList(SimulationManualProjectParam param) {
        if(isEmpty(param.getId())){
            return new ResponseBodyVO(ResponseBodyVO.Response.CLIENT_FAILURE);
        }
        //查询任务信息
        ProjectTaskParam projectTaskParam = new ProjectTaskParam();
        projectTaskParam.setPId(param.getId());
        setPage(param.getCurrentPage()==null?1:param.getCurrentPage(), param.getPageSize()==null?10:param.getPageSize());
        List<ManualProjectTaskVo> manualProjectTaskVos = simulationProjectMapper.selectProjectTaskByProjectId(projectTaskParam);

        PageInfo<ManualProjectTaskVo> pageInfo = new PageInfo(manualProjectTaskVos);

        if(pageInfo.getList() != null){
            for(ManualProjectTaskVo task : pageInfo.getList()){
                task.setRunStartTimeFmt(getRqStr(task.getRunStartTime(), 2));
                task.setRunEndTimeFmt(getRqStr(task.getRunEndTime(), 2));
                setUpSceneInfo(task);

            }
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,pageInfo);
    }

    private void analysisCsvFile(String filePath, int[] analysisFields) throws Exception {

        File file = new File(filePath);
        InputStream fileInputStream = new FileInputStream(file);
        InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream,"utf-8");
        BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
        String line;
        while((line = bufferedReader.readLine()) != null){
            String[] split = line.split(",");
            for(int fs : analysisFields){
                System.out.println(split[fs]);
            }

        }
    }

    /**
     * 根据场景id和场景类型获取场景名
     * @return
     */
    private SceneBaseInfoVo getsceneNameAndOther(String sceneId, String sceneType){
        SceneBaseInfoVo resultVo = new SceneBaseInfoVo();
        if(DictConstants.SCENE_NATURAL.equals(sceneType)){
            SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(sceneId);
            if(sceneBaseInfoVo != null){
                resultVo.setCommonSceneName(sceneBaseInfoVo.getNaturalName());
            }
        }else if(DictConstants.SCENE_STANDARD.equals(sceneType)){
            SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(sceneId);
            if(sceneBaseInfoVo != null){
                resultVo.setCommonSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_ACCIDENT.equals(sceneType)){
            SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(sceneId);
            if(sceneBaseInfoVo != null){
                resultVo.setCommonSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_GENERAL.equals(sceneType)){
            ///TODO 泛化场景暂不支持

        }

        return resultVo;

    }

    private void setAlgorithmDropDown(List<DropDownTypeVo> result){
        List<AlgorithmBaseInfoVo> algorithmBaseInfoVo = simulationProjectMapper.selectAlgorithmBaseInfoById(null);
        List<DropDownVo> algorithmList = new ArrayList<>();
        for(AlgorithmBaseInfoVo v : algorithmBaseInfoVo){
            DropDownVo dropDownVo = new DropDownVo();
            dropDownVo.setId(v.getId());
            dropDownVo.setName(v.getAlgorithmName());
            algorithmList.add(dropDownVo);
        }
        DropDownTypeVo algorithmDropDown = new DropDownTypeVo();
        algorithmDropDown.setDropDownList(algorithmList);
        algorithmDropDown.setType("1");
        result.add(algorithmDropDown);
    }
    private void setVehicleDropDown(List<DropDownTypeVo> result){
        List<ConfigVehicleVO> vehicleBaseInfoVo = simulationProjectMapper.selectConfigVehicle(null);
        List<DropDownVo> vehicleList = new ArrayList<>();
        for(ConfigVehicleVO v : vehicleBaseInfoVo){
            DropDownVo dropDownVo = new DropDownVo();
            dropDownVo.setId(v.getId());
            dropDownVo.setName(v.getConfigName());

            //获取传感器信息
            List<ConfigSensorVo> configSensorVos = simulationProjectMapper.selectConfigSensor(v.getId());
            String sensor = "";
            if(!isEmpty(configSensorVos) && configSensorVos.get(0) != null){
                StringBuffer stringBuffer = new StringBuffer();
                for(ConfigSensorVo cv : configSensorVos){
                    stringBuffer.append(cv.getSensorType()+",");
                }
                sensor = stringBuffer.substring(0,stringBuffer.lastIndexOf(","));

            }
            dropDownVo.setSensor(sensor);

            vehicleList.add(dropDownVo);
        }

        DropDownTypeVo vehicleDropDown = new DropDownTypeVo();
        vehicleDropDown.setDropDownList(vehicleList);
        vehicleDropDown.setType("2");
        result.add(vehicleDropDown);
    }
    private void setScenePackageDropDown(List<DropDownTypeVo> result){
        List<ScenePackageBaseVo> scenePackageBaseVo = simulationProjectMapper.selectScenePackageBaseById(null);
        List<DropDownVo> scenePackageList = new ArrayList<>();
        for(ScenePackageBaseVo v : scenePackageBaseVo){
            DropDownVo dropDownVo = new DropDownVo();
            dropDownVo.setId(v.getPackageId());
            dropDownVo.setName(v.getPackageName());
            dropDownVo.setSceneNum(v.getSceneNum());
            scenePackageList.add(dropDownVo);
        }
        DropDownTypeVo scenePackageDropDown = new DropDownTypeVo();
        scenePackageDropDown.setDropDownList(scenePackageList);
        scenePackageDropDown.setType("3");
        result.add(scenePackageDropDown);
    }


    private SimulationManualProjectPo convertParamToPo(SimulationManualProjectParam param){
        SimulationManualProjectPo po = new SimulationManualProjectPo();
        po.setId(param.getId());
        po.setProjectName(param.getProjectName());
        po.setProjectDescribe(param.getProjectDescribe());
        po.setAlgorithm(param.getAlgorithm());
        po.setVehicle(param.getVehicle());
        po.setScene(param.getScene());
        //po.setOperationCycle(param.getOperationCycle());
        po.setMaxSimulationTime(param.getMaxSimulationTime());
        po.setParallelism(param.getParallelism());
        //po.setRuleView(param.getRuleView());
        po.setIsChoiceGpu(param.getIsChoiceGpu());
        po.setNowRunState(param.getNowRunState());
        po.setAlgorithmType(param.getAlgorithmType());
        return po;

    }

    private SimulationManualProjectVo convertPoToVo(SimulationManualProjectVo vo){
        Date createTime = vo.getCreateTime();
        vo.setCreateTimeFmt(getRqStr(createTime, 2));
        String algorithm = vo.getAlgorithm();
        List<AlgorithmBaseInfoVo> algorithmBaseInfoVos = simulationProjectMapper.selectAlgorithmBaseInfoById(algorithm);
        if(!isEmpty(algorithmBaseInfoVos)){
            AlgorithmBaseInfoVo algorithmBaseInfoVo = algorithmBaseInfoVos.get(0);
            vo.setAlgorithm(algorithmBaseInfoVo.getAlgorithmName());
        }
        vo.setNowRunStateDict(getDictName(DictConstants.PROJECT_RUN_STATE, vo.getNowRunState()));
        vo.setEvaluationLevelDict(getDictName(DictConstants.EVALUATION_LEVEL,vo.getEvaluationLevel()));
        return vo;
    }

    private void convertPoToVo (SimulationManualProjectPo po, SimulationManualProjectSingleVo vo){
        vo.setId(po.getId());
        vo.setProjectName(po.getProjectName());
        vo.setProjectDescribe(po.getProjectDescribe());
        vo.setAlgorithm(po.getAlgorithm());
        vo.setVehicle(po.getVehicle());
        vo.setScene(po.getScene());
        vo.setMaxSimulationTime(po.getMaxSimulationTime());
        vo.setParallelism(po.getParallelism());
        vo.setIsChoiceGpu(po.getIsChoiceGpu());

    }

    private void createProjectId(SimulationManualProjectPo po){
        Integer nowRq = getRq(null,0);
        po.setProjectDate(nowRq);
        SimulationManualProjectPo po1 = simulationProjectMapper.selectLastProjectId(nowRq);
        if(po1 == null){
            //生成新id
            po.setProjectNum(1);
        }else{
            po.setProjectNum(po1.getProjectNum()+1);
        }
        po.setProjectId(po.getProjectDate()+"-"+po.getProjectNum());

    }

    private Integer getRq(Date date, int index){
        return  TimeUtil.getRq(date,index);
    }

    private String getRqStr(Date date, int index){
        SimpleDateFormat sdf = new SimpleDateFormat(dateFmtArr[index]);
        if(date == null){
            return "";
        }
        return sdf.format(date);

    }

    private Date getDate(String dateFmt, int index){
        SimpleDateFormat sdf = new SimpleDateFormat(dateFmtArr[index]);
        try {
            return sdf.parse(dateFmt);
        } catch (ParseException e) {
            return null;
        }
    }

    private boolean isEmpty(String value){
        if(value == null){
            return true;
        }
        value = value.trim();
        if(value.length() == 0){
            return true;
        }
        return false;
    }
    private boolean isEmpty(List list){
        if(list == null || list.size() <= 0){
            return true;
        }
        return false;
    }

    private void setPage(Integer pageNum, Integer pageSize){
        PageVO pageVO = new PageVO();
        pageVO.setCurrentPage(pageNum);
        pageVO.setPageSize(pageSize);
        PageUtil.setPageInfo(pageVO);
    }

    private Double saveTwoDecimalPlaces(Double d){
        if(d == null){
            return null;
        }
        return new BigDecimal(d).setScale(2, RoundingMode.UP).doubleValue();
    }

    /**
     * 设置场景基本字段
     */
    private void setUpSceneInfo(ManualProjectTaskVo vo){
        String sceneType = vo.getSceneType();
        String sceneId = vo.getSceneId();
        SceneBaseInfoVo sceneBaseInfoVo;
        if(DictConstants.SCENE_NATURAL.equals(sceneType)){
            //自然驾驶
            sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(sceneId);
            if(sceneBaseInfoVo != null){
                vo.setSceneName(sceneBaseInfoVo.getNaturalName());
            }
        }else if(DictConstants.SCENE_STANDARD.equals(sceneType)){
            //标准法规
            sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(sceneId);
            if(sceneBaseInfoVo != null){
                vo.setSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_ACCIDENT.equals(sceneType)){
            //交通事故
            sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(sceneId);
            if(sceneBaseInfoVo != null){
                vo.setSceneName(sceneBaseInfoVo.getSceneName());
            }
        }else if(DictConstants.SCENE_GENERAL.equals(sceneType)){
            /// TODO 泛化场景暂不支持

        }
    }

    private ManualProjectTaskVo convertManualProjectTaskPoToVo(ManualProjectTaskPo po){
        ManualProjectTaskVo manualProjectTaskVo = new ManualProjectTaskVo();
        manualProjectTaskVo.setId(po.getId());
        manualProjectTaskVo.setPId(po.getPId());
        manualProjectTaskVo.setSceneId(po.getSceneId());
//        manualProjectTaskVo.setSceneName(po.getSceneName());
        manualProjectTaskVo.setSceneType(po.getSceneType());
        manualProjectTaskVo.setRunStartTimeFmt(getRqStr(po.getRunStartTime(), 2));
        manualProjectTaskVo.setRunEndTimeFmt(getRqStr(po.getRunEndTime(), 2));
        manualProjectTaskVo.setRunState(po.getRunState());
        manualProjectTaskVo.setRunResult(po.getRunResult());
        return manualProjectTaskVo;
    }

    /**
     * 生成项目场景信息和最后一级指标信息 TODO 暂不使用
     */
    private void createProjectSceneAndTarget(String id){
        //1.查询场景包下最后一级指标和指标下所有场景
        List<ScenePackageSubListVO> scenePackageSubListVOS = simulationProjectMapper.selectSubListByPid(id);

        //2.保存最后一级指标
        for(ScenePackageSubListVO v : scenePackageSubListVOS){
            SimulationMptLastTargetScorePo simulationMptTargetScorePo = new SimulationMptLastTargetScorePo();
            simulationMptTargetScorePo.setPId(id);
            simulationMptTargetScorePo.setTarget(v.getSublistId());
            simulationMptTargetScorePo.createPo(AuthUtil.getCurrentUserId());
            simulationProjectMapper.insertSimulationMptLastTargetScorePo(simulationMptTargetScorePo);

            //3.保存场景
            String sceneNaturalIds = v.getSceneNaturalIds();
            String sceneTrafficIds = v.getSceneTrafficIds();
            String sceneStatueIds = v.getSceneStatueIds();
            if(!isEmpty(sceneNaturalIds)){
                String[] split = sceneNaturalIds.split(",");
                for(String s : split){
                    SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(s);
                    SimulationMptSceneScorePo simulationMptSceneScorePo = new SimulationMptSceneScorePo();
                    simulationMptSceneScorePo.setPId(id);
                    simulationMptSceneScorePo.setSceneId(s);
                    simulationMptSceneScorePo.setSceneType(DictConstants.SCENE_NATURAL);
                    simulationMptSceneScorePo.createPo(AuthUtil.getCurrentUserId());
                    simulationProjectMapper.insertSimulationMptSceneScorePo(simulationMptSceneScorePo);
                }

            }
            if(!isEmpty(sceneTrafficIds)){
                String[] split = sceneTrafficIds.split(",");
                for(String s : split){
                    SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(s);
                    SimulationMptSceneScorePo simulationMptSceneScorePo = new SimulationMptSceneScorePo();
                    simulationMptSceneScorePo.setPId(id);
                    simulationMptSceneScorePo.setSceneId(s);
                    simulationMptSceneScorePo.setSceneType(DictConstants.SCENE_ACCIDENT);
                    simulationMptSceneScorePo.createPo(AuthUtil.getCurrentUserId());
                    simulationProjectMapper.insertSimulationMptSceneScorePo(simulationMptSceneScorePo);
                }
            }
            if(!isEmpty(sceneStatueIds)){
                String[] split = sceneStatueIds.split(",");
                for(String s : split){
                    SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(s);
                    SimulationMptSceneScorePo simulationMptSceneScorePo = new SimulationMptSceneScorePo();
                    simulationMptSceneScorePo.setPId(id);
                    simulationMptSceneScorePo.setSceneId(s);
                    simulationMptSceneScorePo.setSceneType(DictConstants.SCENE_STANDARD);
                    simulationMptSceneScorePo.createPo(AuthUtil.getCurrentUserId());
                    simulationProjectMapper.insertSimulationMptSceneScorePo(simulationMptSceneScorePo);
                }
            }

            ///TODO 泛化场景暂不支持
        }

        //4.查询第一级指标

        //5.保存第一级指标

    }


    /**
     * 级联获取场景指标得分数据
     * @param parentVoList 首次传null
     * @param isRoot 首次传true
     * @param scenePackageId 场景包id
     * @param pId 项目表id，用于统计得分使用
     * @param level 指标级别
     * @return
     */
    private List<ScenePackageSubListVO> selectScenePackageSubListTreeAndSetScore(List<ScenePackageSubListVO> parentVoList,
                                                                      boolean isRoot,String scenePackageId, String pId, Integer level){

        if(isRoot){
            //查找一级节点
            parentVoList = simulationProjectMapper.selectSubSceneByPid(scenePackageId);
            if(!isEmpty(parentVoList)){
                selectScenePackageSubListTreeAndSetScore(parentVoList, false, null,pId,level);
            }

        }else{

            //获取子节点集合
            for(ScenePackageSubListVO pvo : parentVoList){

                pvo.setLevel(level);

                //二级指标获取总分
                if(level == 2){
                    setFirstTargetScore(pvo,pId);
                }

                List<ScenePackageSubListVO> cvoList = simulationProjectMapper.selectSubSceneByPid(pvo.getSublistId());
                if(!isEmpty(cvoList)){
                    //存入父节点集合中
                    pvo.setChildScenePackageSubListVOList(cvoList);
                    //继续查找下一节点
                    selectScenePackageSubListTreeAndSetScore(cvoList,false, null, pId, level+1);
                }else{

                    //没有子节点；最后一级，获取指标得分,指标得分说明,和指标下场景得分
                    setScore(pvo,pId);
                }

            }
        }


        return parentVoList;
    }


    private Map<String, Object> selectSceneScore(String scenePackageId, String projectId) {

        //查询场景包所有数据
        List<SublistScoreVo> pos = simulationProjectMapper.selectSubScore(scenePackageId);

        HashMap<String, Object> hashMap = new HashMap<>();

        //1.获取项目下所有场景得分
        ProjectTaskParam param = new ProjectTaskParam();
        param.setPId(projectId);
        List<ManualProjectTaskVo> manualProjectTaskVos = simulationProjectMapper.selectProjectTaskByProjectId(param);

        List<SublistScoreVo> lastSubList = new ArrayList<>();
        //2.拼接场景得分信息
        for(SublistScoreVo po : pos){
            if(ObjectUtil.isNotNull(po.getPackageAndRules())){

                //获取场景得分信息
                for(ManualProjectTaskVo sc : manualProjectTaskVos)
                    if (sc.getLastTargerId().equals(po.getId())) {
                        SublistScoreVo sublistScoreVo = new SublistScoreVo();
                        BeanUtils.copyProperties(po, sublistScoreVo);

                        sublistScoreVo.setSceneScore(sc.getScore()); //得分
                        sublistScoreVo.setTargetEvaluate(sc.getTargetEvaluate());//指标评价
                        sublistScoreVo.setReturnSceneId(sc.getReturnSceneId());//显示的场景id
                        String sceneType = sc.getSceneType();
                        if(DictConstants.SCENE_NATURAL.equals(sceneType)){
                            //自然驾驶
                            sublistScoreVo.setSceneType("自然驾驶");
                        }else if(DictConstants.SCENE_STANDARD.equals(sceneType)){
                            //标准法规
                            sublistScoreVo.setSceneType("标准法规");
                        }else if(DictConstants.SCENE_ACCIDENT.equals(sceneType)){
                            //交通事故
                            sublistScoreVo.setSceneType("交通事故");
                        }else if(DictConstants.SCENE_GENERAL.equals(sceneType)){
                            /// TODO 泛化场景暂不支持

                        }

                        lastSubList.add(sublistScoreVo);
                    }

            }
        }

        int size = 0;
        List<List<String>> result = new ArrayList<>();
        //末级指标
        for(SublistScoreVo sp : lastSubList){
            setParentSub(sp, null, pos);
            String sublistName = sp.getSublistName();
            sublistName += ","+sp.getReturnSceneId()+","+sp.getSceneType()+","+sp.getSceneScore()+","+sp.getTargetEvaluate()+","+sp.getScoreExplain();

            String[] split = sublistName.split(",");

            List<String> strings = new LinkedList<>();
            strings.addAll(Arrays.asList(split));
            if(size<strings.size()){
                size = strings.size();
            }
            result.add(strings);

        }

        List<SceneScListVo> objects = new ArrayList<>();
        //结果补全
        for(List<String> list : result){
            int offset = size - list.size();
            int start = list.size() - 5;
//            for(int i=0;i<offset;i++){
//                list.add(start+i,"");
//            }
            SceneScListVo sceneScListVo = new SceneScListVo();
            for(int i=0;i<start;i++){
                if(0 == i){
                    sceneScListVo.setSublistName1(list.get(i));
                }else if(1 == i){
                    sceneScListVo.setSublistName2(list.get(i));
                }else if(2 == i){
                    sceneScListVo.setSublistName3(list.get(i));
                }else if(3 == i){
                    sceneScListVo.setSublistName4(list.get(i));
                }else if(4 == i){
                    sceneScListVo.setSublistName5(list.get(i));
                }else if(5 == i){
                    sceneScListVo.setSublistName6(list.get(i));
                }

            }

            sceneScListVo.setSceneId(list.get(start));
            sceneScListVo.setSceneIdType(list.get(start+1));
            sceneScListVo.setSceneScore(list.get(start+2));
            sceneScListVo.setTargetEvaluate(list.get(start+3));
            sceneScListVo.setScoreExplain(list.get(start+4));

            objects.add(sceneScListVo);
        }

        List<Map<String, String>> cloums = new ArrayList<>();
        for(int i=0; i<size-5;i++){
            setCloums(i,cloums);
            /*if(0 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","一级指标");
                map.put("prop","sublistName1");
                cloums.add(map);
            }else if(1 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","二级指标");
                map.put("prop","sublistName2");
                cloums.add(map);
            }else if(2 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","三级指标");
                map.put("prop","sublistName3");
                cloums.add(map);
            }else if(3 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","四级指标");
                map.put("prop","sublistName4");
                cloums.add(map);
            }else if(4 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","五级指标");
                map.put("prop","sublistName5");
                cloums.add(map);
            }else if(5 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","六级指标");
                map.put("prop","sublistName6");
                cloums.add(map);
            }*/
        }

        Map<String, String> map1 = new HashMap<>();
        map1.put("label","场景ID");
        map1.put("prop","sceneId");
        cloums.add(map1);

        Map<String, String> map2 = new HashMap<>();
        map2.put("label","场景类型");
        map2.put("prop","sceneIdType");
        cloums.add(map2);

        Map<String, String> map3 = new HashMap<>();
        map3.put("label","测试得分");
        map3.put("prop","sceneScore");
        cloums.add(map3);

        Map<String, String> map4 = new HashMap<>();
        map4.put("label","指标评价");
        map4.put("prop","targetEvaluate");
        cloums.add(map4);

        Map<String, String> map5 = new HashMap<>();
        map5.put("label","得分说明");
        map5.put("prop","scoreExplain");
        cloums.add(map5);

        hashMap.put("cloums",cloums);
        hashMap.put("result",objects);

        return hashMap;


    }

    /**
     *
     * @param po 末级指标
     * @param parentPo 上级指标
     * @param pos 所有指标
     */
    private void setParentSub(SublistScoreVo po, SublistScoreVo parentPo, List<SublistScoreVo> pos) {
        /*for(SublistScoreVo p : pos){
            if(po.getParentId().equals(p.getId())){
                po.setSublistName(p.getSublistName()+","+po.getSublistName());
                setParentSub(p, pos);
            }

        }*/
        if(parentPo == null){
            for(SublistScoreVo p : pos){
                setSc(po,p,po,pos);
            }

        }else{
            for(SublistScoreVo p : pos){
                setSc(parentPo,p,po,pos);
            }
        }
    }

    /**
     *
     * @param p1 上级指标
     * @param p2 比对指标
     * @param p3 末级指标
     * @param pos 所有指标
     */
    private void setSc(SublistScoreVo p1, SublistScoreVo p2, SublistScoreVo p3, List<SublistScoreVo> pos){
        //上级指标
        if(p1.getParentId().equals(p2.getId())){

            //末级指标拼接指标名
            p3.setSublistName(p2.getSublistName()+","+p3.getSublistName());
            //查找上级指标，继续拼接
            setParentSub(p3,p2, pos);
        }


    }




    private Map<String,Object> selectScenePackageSubListAndSetScore(String scenePackageId){

        //查询场景包所有数据
        List<SublistScoreVo> pos = simulationProjectMapper.selectSubScore(scenePackageId);

        HashMap<String, Object> hashMap = new HashMap<>();

        int size = 0;
        List<List<String>> result = new ArrayList<>();
        //递归指标名称得分
        for(SublistScoreVo po : pos){
            if(ObjectUtil.isNotNull(po.getPackageAndRules())){
                //末级指标
                setParent(po,null,pos,scenePackageId);

                String sublistName = po.getSublistName();
                String firstScore = po.getFirScore();//first得分
                String LastScore = po.getLasScore();//last得分
                String sceneNum = po.getSceneNum();//场景数量
                String notStandardSceneNum = po.getNotStandardSceneNum();//未达标场景数量

                if(ObjectUtil.isNotNull(sublistName)){
                    sublistName += ","+sceneNum+","+notStandardSceneNum+","+LastScore+","+firstScore;
                    String[] split = sublistName.split(",");


                    List<String> strings = new LinkedList<>();
                    strings.addAll(Arrays.asList(split));
                    if(size<strings.size()){
                        size = strings.size();
                    }
                    result.add(strings);
                }

            }

        }


        List<SubScListVo> objects = new ArrayList<>();
        //结果补全
        for(List<String> list : result){
            int offset = size - list.size();
            int start = list.size() - 4;
            /*for(int i=0;i<offset;i++){
                list.add(start+i,"");
            }*/
            SubScListVo subScListVo = new SubScListVo();
            for(int i=0;i<start;i++){
                if(0 == i){
                    subScListVo.setSublistName1(list.get(i));
                }else if(1 == i){
                    subScListVo.setSublistName2(list.get(i));
                }else if(2 == i){
                    subScListVo.setSublistName3(list.get(i));
                }else if(3 == i){
                    subScListVo.setSublistName4(list.get(i));
                }else if(4 == i){
                    subScListVo.setSublistName5(list.get(i));
                }else if(5 == i){
                    subScListVo.setSublistName6(list.get(i));
                }

            }
            subScListVo.setSceneNum(list.get(start));
            subScListVo.setNotStandardSceneNum(list.get(start+1));
            subScListVo.setLastScore(list.get(start+2));
            subScListVo.setFirstScore(list.get(start+3));

            objects.add(subScListVo);


        }

        List<Map<String, String>> cloums = new ArrayList<>();
        for(int i=0; i<size-4;i++){
            setCloums(i,cloums);
            /*if(0 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","一级指标");
                map.put("prop","sublistName1");
                cloums.add(map);
            }else if(1 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","二级指标");
                map.put("prop","sublistName2");
                cloums.add(map);
            }else if(2 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","三级指标");
                map.put("prop","sublistName3");
                cloums.add(map);
            }else if(3 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","四级指标");
                map.put("prop","sublistName4");
                cloums.add(map);
            }else if(4 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","五级指标");
                map.put("prop","sublistName5");
                cloums.add(map);
            }else if(5 == i){
                Map<String, String> map = new HashMap<>();
                map.put("label","六级指标");
                map.put("prop","sublistName6");
                cloums.add(map);
            }*/
        }
        Map<String, String> map1 = new HashMap<>();
        map1.put("label","场景数量");
        map1.put("prop","sceneNum");
        cloums.add(map1);

        Map<String, String> map2 = new HashMap<>();
        map2.put("label","未达标场景数量");
        map2.put("prop","notStandardSceneNum");
        cloums.add(map2);

        Map<String, String> map3 = new HashMap<>();
        map3.put("label","得分");
        map3.put("prop","lastScore");
        cloums.add(map3);

        Map<String, String> map4 = new HashMap<>();
        map4.put("label","总分");
        map4.put("prop","firstScore");
        cloums.add(map4);

        hashMap.put("cloums",cloums);
        hashMap.put("result",objects);


        return hashMap;


    }

    /**
     *
     * @param po 末级指标
     * @param pos 场景包下所有指标
     * @param rootId 场景包id，用于判断是否根指标
     */
    private void setParent(SublistScoreVo po, List<SublistScoreVo> pos, String rootId) {
        for(SublistScoreVo p : pos){

            if(po.getParentId().equals(p.getId())){
                //给一级指标添加末级指标得分,给末级指标添加一级指标得分
                if(p.getParentId().equals(rootId)){
                    p.setLasScore(po.getLasScore());
                    po.setFirScore(p.getFirScore());
                }
                po.setSublistName(p.getSublistName()+","+po.getSublistName());
                setParent(p, pos, rootId);
            }

        }
    }

    /**
     *
     * @param po 末级指标
     * @param pos 场景包下所有指标
     * @param rootId 场景包id，用于判断是否一级指标
     */
    private void setParent(SublistScoreVo po, SublistScoreVo parentPo, List<SublistScoreVo> pos, String rootId) {
        //第一次查找
        if(parentPo == null){
            for(SublistScoreVo p : pos){

                setSubSc(po,p,po,pos,rootId);
/*                if(po.getParentId().equals(p.getId())){
                    //根指标给一级指标添加末级指标得分,给末级指标添加一级指标得分
                    if(p.getParentId().equals(rootId)){
                        p.setLasScore(po.getLasScore());
                        po.setFirScore(p.getFirScore());
                    }
                    //末级指标拼接指标名
                    po.setSublistName(p.getSublistName()+","+po.getSublistName());
                    //查找上级指标，继续拼接
                    setParent(po,p, pos, rootId);

                }*/
            }

        }else{
            for(SublistScoreVo p : pos){

                setSubSc(parentPo,p,po,pos,rootId);

/*                //上级指标
                if(parentPo.getParentId().equals(p.getId())){
                    //根指标给一级指标添加末级指标得分,给末级指标添加一级指标得分
                    if(p.getParentId().equals(rootId)){
                        p.setLasScore(po.getLasScore());
                        po.setFirScore(p.getFirScore());
                    }
                    //末级指标拼接指标名
                    po.setSublistName(p.getSublistName()+","+po.getSublistName());
                    //查找上级指标，继续拼接
                    setParent(po,p, pos, rootId);
                }*/

            }
        }

    }


    /**
     *
     * @param p1 上级指标
     * @param p2 比对指标
     * @param p3 末级指标
     * @param pos 所有指标
     * @param rootId 场景包id
     */
    private void setSubSc(SublistScoreVo p1, SublistScoreVo p2, SublistScoreVo p3, List<SublistScoreVo> pos, String rootId){
        //上级指标
        if(p1.getParentId().equals(p2.getId())){
            //根指标给一级指标添加末级指标得分,给末级指标添加一级指标得分
            if(p2.getParentId().equals(rootId)){
                p2.setLasScore(p3.getLasScore());
                p3.setFirScore(p2.getFirScore());
            }
            //末级指标拼接指标名
            p3.setSublistName(p2.getSublistName()+","+p3.getSublistName());
            //查找上级指标，继续拼接
            setParent(p3,p2, pos, rootId);
        }


    }


/*
    *//**
     * 查询所有指标和得分
     * @param
     *//*
    private void selectScenePackageSubListAndSetScore(String pId, String scenePackageId, List<List<String>> resultArr, List<ScenePackageSubListVO>  single){

        //首次，初始化数据以及赋值
        if(isEmpty(single)){
            //单个级联，按最后一级到第一级指标顺序添加
            single= new ArrayList<>();
            //查询所有指标最后一级
            List<ScenePackageSubListVO> vos = simulationProjectMapper.selectSubListByPid(scenePackageId);
            for(ScenePackageSubListVO vo : vos){
                selectSceneSubList(vo, pId, single, resultArr);
            }
        }
        //继续添加数据
        else{
            //查询父级节点，只有一个
            ScenePackageSubListVO vo = simulationProjectMapper.selectsublistBySublistId(scenePackageId);
            //如果是根节点，不再查询，组装返回数据
            if(vo.getSublistId().equals(scenePackageId)){
                if(isEmpty(resultArr)){
                    //初始化返回数据
                    resultArr = new ArrayList<>();
                    List<String> singleList = new ArrayList<>();
                    //反转list,正序添加（从一级开始）
                    Collections.reverse(single);

                    for(int i=0;i<single.size();i--){
                        if(i == single.size()-1){
                            singleList.add(String.valueOf(single.get(i).getSceneNum()));
                            singleList.add(String.valueOf(single.get(i).getNotStandardSceneNum()));
                            singleList.add(String.valueOf(single.get(i).getScore()));

                            singleList.add(String.valueOf(single.get(i).getScore()));

                        }
                        singleList.add(single.get(i).getSublistName());
                    }






                }

            }else{
                selectSceneSubList(vo, pId, single, resultArr);
            }
        }

    };*/

/*    private void selectSceneSubList(ScenePackageSubListVO vo, String pId, List<ScenePackageSubListVO>  single, List<List<String>> resultArr){
        //获取未达标个数，得分，总分
        setScore(vo,pId);
        //放入集合
        single.add(vo);
        //递归查询上级指标
        selectScenePackageSubListAndSetScore(pId, vo.getParentId(), resultArr, single);
    }*/

    private void setScore(ScenePackageSubListVO vo, String pId){
        setLastTargetScore(vo,pId);
        setSceneScore(vo,pId);
    }

    private void setSceneScore(ScenePackageSubListVO vo, String pId){
        String sceneNaturalIds = vo.getSceneNaturalIds();
        String sceneTrafficIds = vo.getSceneTrafficIds();
        String sceneStatueIds = vo.getSceneStatueIds();
        List<SceneScoreVo> sceneScoreVos = new ArrayList<>();
        List<SceneScoreVo> NaturalSceneScoreVos = setSceneScore(vo, sceneNaturalIds, DictConstants.SCENE_NATURAL, pId);
        List<SceneScoreVo> StatueSceneScoreVos = setSceneScore(vo, sceneStatueIds, DictConstants.SCENE_STANDARD, pId);
        List<SceneScoreVo> TrafficSceneScoreVos = setSceneScore(vo, sceneTrafficIds, DictConstants.SCENE_ACCIDENT, pId);
        ///TODO 暂不支持泛化场景

        //合成一个list
        if(!isEmpty(NaturalSceneScoreVos)){
            sceneScoreVos.addAll(NaturalSceneScoreVos);
        }
        if(!isEmpty(StatueSceneScoreVos)){
            sceneScoreVos.addAll(StatueSceneScoreVos);
        }
        if(!isEmpty(TrafficSceneScoreVos)){
            sceneScoreVos.addAll(TrafficSceneScoreVos);
        }
        vo.setSceneScoreList(sceneScoreVos);
    }
    private void setLastTargetScore(ScenePackageSubListVO vo, String pId){
        SimulationMptLastTargetScorePo poParam = new SimulationMptLastTargetScorePo();
        poParam.setPId(pId);
        poParam.setTarget(vo.getSublistId());
        SimulationMptLastTargetScorePo po = simulationProjectMapper.selectLastTargetScore(poParam);
        if(po != null){
            vo.setNotStandardSceneNum(po.getNotStandardSceneNum());
            vo.setScore(po.getScore());
            vo.setScoreExplain(po.getScoreExplain());
        }

    }

    private void setFirstTargetScore(ScenePackageSubListVO vo, String pId){
        SimulationMptFirstTargetScorePo poParam = new SimulationMptFirstTargetScorePo();
        poParam.setPId(pId);
        poParam.setTarget(vo.getSublistId());
        SimulationMptFirstTargetScorePo po = simulationProjectMapper.selectFirstTargetScore(poParam);
        if(po != null){
            vo.setScore(po.getScore());
        }

    }

    private List<SceneScoreVo> setSceneScore(ScenePackageSubListVO vo,String sceneIds, String sceneType, String pId){
        if(isEmpty(sceneIds)){
            return null;
        }
        String[] sceneIdArr = sceneIds.split(",");
        SceneScoreParam sceneScoreParam = new SceneScoreParam();
        sceneScoreParam.setPId(pId);
        sceneScoreParam.setLastTargerId(vo.getSublistId());
        sceneScoreParam.setSceneType(sceneType);
        sceneScoreParam.setSceneIds(Arrays.asList(sceneIdArr));
        List<SceneScoreVo> sceneScoreVos = simulationProjectMapper.selectSceneScoreByIds(sceneScoreParam);

        for(SceneScoreVo s : sceneScoreVos){
            /*if(SceneTypeEnum.SCENE_NATURAL.getCode().equals(sceneType)){
                SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneNatural(s.getSceneId());
                if(sceneBaseInfoVo != null){
                    s.setSceneName(sceneBaseInfoVo.getNaturalName());
                }
            }else if(SceneTypeEnum.SCENE_STANDARD.getCode().equals(sceneType)){
                SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneStandardsRegulations(s.getSceneId());
                if(sceneBaseInfoVo != null){
                    s.setSceneName(sceneBaseInfoVo.getSceneName());
                }
            }else if(SceneTypeEnum.SCENE_ACCIDENT.getCode().equals(sceneType)){
                SceneBaseInfoVo sceneBaseInfoVo = simulationProjectMapper.selectSceneAccidentById(s.getSceneId());
                if(sceneBaseInfoVo != null){
                    s.setSceneName(sceneBaseInfoVo.getSceneName());
                }
            }else if(SceneTypeEnum.SCENE_GENERAL.getCode().equals(sceneType)){
                ///TODO 泛化场景暂不支持

            }*/
            s.setSceneType(SceneTypeEnum.getState(sceneType));
        }

        return sceneScoreVos;

    }

    private String getEvaluationLevelReport(Double algorithmScore){
        String evaluationLevelReport = "";
        if(algorithmScore >= 90){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_G;
        }else if(algorithmScore >= 80 && algorithmScore < 90){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_A;
        }else if(algorithmScore >= 70 && algorithmScore < 80){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_M;
        }else if(algorithmScore < 70){
            evaluationLevelReport = DictConstants.REPORT_LEVEL_P;
        }
        return evaluationLevelReport;
    }

    /**
     * 获取二级场景下所有得分不为0的场景数量
     * @param sublistId
     * @param pId
     * @return
     */
    private Integer getSetScoreNum(String sublistId,String pId){
        SceneScoreVo vo = selectScoreNum(null, null, true, pId, sublistId);
        return vo.getNum();
    }


    /**
     * 获取一级指标下所有得分不为0的场景总数
     * @param resultVo 返回值，返回统计数量，首次传null
     * @param parentVoList 首次传null
     * @param isRoot 首次传true
     * @param pId 项目表id，用于统计得分使用
     * @param sublistId 指标id（用户查询指标下的所有场景）
     * @return
     */
    private SceneScoreVo selectScoreNum(SceneScoreVo resultVo, List<ScenePackageSubListVO> parentVoList,
                                                                                 boolean isRoot, String pId,String sublistId){
        if(isRoot){
            //初始化返回值
            resultVo = new SceneScoreVo();
            resultVo.setNum(0);

            //查找下一级指标
            parentVoList = simulationProjectMapper.selectSubSceneByPid(sublistId);
            if(!isEmpty(parentVoList)){
                selectScoreNum(resultVo, parentVoList, false, pId,null);
            }else{
                //当前指标为最后一级指标，获取当前指标，统计得分场景数
                setScoreNum(sublistId, pId, resultVo);
            }

        }else{

            //获取子节点集合
            for(ScenePackageSubListVO pvo : parentVoList){

                List<ScenePackageSubListVO> cvoList = simulationProjectMapper.selectSubSceneByPid(pvo.getSublistId());
                if(!isEmpty(cvoList)){
                    //存入父节点集合中
                    pvo.setChildScenePackageSubListVOList(cvoList);
                    //继续查找下一节点
                    selectScoreNum(resultVo, cvoList,false, pId, null);
                }else{
                    //没有子节点；获取最后一级的得分场景数量
                    setScoreNum(pvo.getSublistId(), pId, resultVo);

                }

            }
        }

        return resultVo;
    }

    private void setScoreNum(String sublistId, String pId, SceneScoreVo resultVo){
//        ScenePackageSubListVO scenePackageSubListVO = simulationProjectMapper.selectsublistBySublistId(sublistId);
        SimulationMptSceneScorePo po = new SimulationMptSceneScorePo();
        po.setPId(pId);
//        po.setLastTargerId(scenePackageSubListVO.getSublistId());
        po.setLastTargerId(sublistId);
        SceneScoreVo sceneScoreVo = simulationProjectMapper.selectSceneScoreNumQuery(po);
        resultVo.setNum(resultVo.getNum()+sceneScoreVo.getNum());
    }


    /**
     * 计算评测等级
     * @return
     */
    private String getEvaluationLevel(SimulationManualProjectPo po){
        /*
        汇总测试得分(计算评级)计算方法：（每一项一级指标的测试得分*测试权重）累加
         */
        List<ScenePackageSubListVO> scenePackageSubListVOS = simulationProjectMapper.selectSubSceneByPid(po.getScene());

        String evaluationLevelReport = "";
        if(!isEmpty(scenePackageSubListVOS)){
            //汇总数据初始化
            Double totalScore = 0D;

            for(ScenePackageSubListVO v : scenePackageSubListVOS){
                String weight = v.getWeight();
                Double weightDouble = Double.valueOf(weight);
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo = new SimulationMptFirstTargetScorePo();
                simulationMptFirstTargetScorePo.setPId(po.getId());
                simulationMptFirstTargetScorePo.setTarget(v.getSublistId());
                //单个二级指标得分
                SimulationMptFirstTargetScorePo simulationMptFirstTargetScorePo1 = simulationProjectMapper.selectFirstTargetScore(simulationMptFirstTargetScorePo);
                if(simulationMptFirstTargetScorePo1 == null){
                    return evaluationLevelReport;
                }
                Double score = simulationMptFirstTargetScorePo1.getScore();
                totalScore += score*(weightDouble/100);
            }

            evaluationLevelReport = getEvaluationLevelReport(saveTwoDecimalPlaces(totalScore));
            return evaluationLevelReport;
        }

        return evaluationLevelReport;

    }

    private void setCloums(Integer i, List<Map<String, String>> cloums){
        if(0 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","一级指标");
            map.put("prop","sublistName1");
            cloums.add(map);
        }else if(1 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","二级指标");
            map.put("prop","sublistName2");
            cloums.add(map);
        }else if(2 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","三级指标");
            map.put("prop","sublistName3");
            cloums.add(map);
        }else if(3 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","四级指标");
            map.put("prop","sublistName4");
            cloums.add(map);
        }else if(4 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","五级指标");
            map.put("prop","sublistName5");
            cloums.add(map);
        }else if(5 == i){
            Map<String, String> map = new HashMap<>();
            map.put("label","六级指标");
            map.put("prop","sublistName6");
            cloums.add(map);
        }
    }
    //运行项目统计-折线图
    @Override
    public ResponseBodyVO selectRunProjectBySy() {
        List<RunProjectVO> list=new ArrayList();
        for(Integer i=6;i>-1;i--) {
            RunProjectVO vo=new RunProjectVO();
            Map<String,Object> params = TimeUtil.getPastTime(i);
            params.put("createUserId", AuthUtil.getCurrentUserId());
            Integer num = simulationProjectMapper.selectRunProjectBySy(params);
            vo.setToDate(params.get("toDate").toString());
            vo.setNum(num);
            list.add(vo);
        }
         return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,list);
    }
    //运行任务统计-折线图
    @Override
    public ResponseBodyVO selectRunTaskBySy() {
        List<RunProjectVO> list=new ArrayList();
        for(Integer i=6;i>-1;i--) {
            RunProjectVO vo=new RunProjectVO();
            Map<String,Object> params = TimeUtil.getPastTime(i);
            params.put("createUserId", AuthUtil.getCurrentUserId());
            Integer num = simulationProjectMapper.selectRunTaskBySy(params);
            vo.setToDate(params.get("toDate").toString());
            vo.setNum(num);
            list.add(vo);
        }
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS,list);
    }
    //任务运行状态统计-饼图
    @Override
    public ResponseBodyVO selectRunTaskByState() {
            Map<String,Object> params =new HashMap<>();
            params.put("createUserId", AuthUtil.getCurrentUserId());
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, simulationProjectMapper.selectRunTaskByState(params));
    }
    //项目运行状态统计-饼图
    @Override
    public ResponseBodyVO selectRunProjectByState() {
        Map<String,Object> params =new HashMap<>();
        params.put("createUserId", AuthUtil.getCurrentUserId());
        //10：未执行，20：执行中，30：执行完成，40：已中止
        List list=simulationProjectMapper.selectRunProjectByState(params);
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, list);
    }
    //评测等级分布-饼图
    @Override
    public ResponseBodyVO selectEvaluationLevel() {
        Map<String,Object> params =new HashMap<>();
        params.put("createUserId", AuthUtil.getCurrentUserId());
        return new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, simulationProjectMapper.selectEvaluationLevel(params));
    }


    private void insertOtherAlgorithm(SimulationManualProjectParam param){
        if(ProjectConstants.SY_ALGORITHM_TYPE.equals(param.getAlgorithmType())){
            AlgorithmPO po = new AlgorithmPO();
            po.setId(param.getAlgorithm());
            po.setAlgorithmName(param.getAlgorithmName());
            po.setUploadMode(ProjectConstants.SY_ALGORITHM_TYPE);
            po.setIsDeleted("0");
            AlgorithmPO po1 = simulationProjectMapper.selectalgorithmByQuery(po);
            if(po1 == null){
                simulationProjectMapper.insertOtherAlgorithm(po);
            }else{
                if(!po1.getAlgorithmName().equals(param.getAlgorithmName())){
                    simulationProjectMapper.updateOtherAlgorithm(po);
                }
            }

        }


    }
}
